import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Modal, Button } from 'antd'

import asyncComponent from '@/utils/asyncComponent'
import { getCardCellForm } from './formconfig'
import { getActionForm, getBaseTableActionForm } from '@/menu/components/share/actioncomponent/formconfig'

import MKEmitter from '@/utils/events.js'
import MenuUtils from '@/utils/utils-custom.js'
import ElementForm from './elementform'
import CreateFunc from '@/templates/zshare/createfunc'
import DragElement from './dragaction'
import './index.scss'

const { confirm } = Modal

const ActionForm = asyncComponent(() => import('@/menu/components/share/actioncomponent/actionform'))
const VerifyCard = asyncComponent(() => import('@/templates/zshare/verifycard'))
const VerifyPrint = asyncComponent(() => import('@/templates/sharecomponent/actioncomponent/verifyprint'))
const VerifyExcelIn = asyncComponent(() => import('@/templates/sharecomponent/actioncomponent/verifyexcelin'))
const VerifyExcelOut = asyncComponent(() => import('@/templates/sharecomponent/actioncomponent/verifyexcelout'))

class CardCellComponent extends Component {
  static propTpyes = {
    cards: PropTypes.object,         // 菜单配置信息
    cardCell: PropTypes.object,
    side: PropTypes.string,
    elements: PropTypes.array,       // 元素集
    updateElement: PropTypes.func    // 菜单配置更新
  }

  state = {
    appType: sessionStorage.getItem('appType'),
    card: null,          // 编辑中元素
    formlist: null,      // 表单信息
    elements: null,      // 按钮组
    visible: false,      // 模态框控制
    actvisible: false,   // 按钮编辑模态框
    profVisible: false,  // 验证信息编辑
    record: null
  }

  /**
   * @description 搜索条件初始化
   */
  UNSAFE_componentWillMount () {
    const { elements } = this.props
    let _elements = fromJS(elements).toJS()

    this.setState({
      elements: _elements.map(item => {
        if (item.btnstyle) { // 兼容
          item.style = item.style || {}
          item.style = {...item.style, ...item.btnstyle}
          delete item.btnstyle
        }

        return item
      })
    })
  }

  componentDidMount () {
    MKEmitter.addListener('submitModal', this.handleSave)
    MKEmitter.addListener('cardAddElement', this.cardAddElement)
    MKEmitter.addListener('cardDelElement', this.cardDelElement)
    MKEmitter.addListener('submitComponentStyle', this.updateComponentStyle)
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState)) || !is(fromJS(this.props.cards), fromJS(nextProps.cards))
  }

  UNSAFE_componentWillReceiveProps(nextProps) {
    if (this.props.side !== nextProps.side) {
      this.setState({
        elements: fromJS(nextProps.elements).toJS()
      })
    }
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('submitModal', this.handleSave)
    MKEmitter.removeListener('cardAddElement', this.cardAddElement)
    MKEmitter.removeListener('cardDelElement', this.cardDelElement)
    MKEmitter.removeListener('submitComponentStyle', this.updateComponentStyle)
  }

  updateComponentStyle = (parentId, keys, style) => {
    const { cardCell } = this.props

    if (cardCell.uuid !== parentId) return

    let _elements = this.state.elements.map(item => {
      if (keys.includes(item.uuid)) {
        return this.resetCardStyle(item, {...item.style, ...style})
      }
      return item
    })

    this.setState({
      elements: _elements
    }, () => {
      this.props.updateElement(_elements)
    })
  }

  cardDelElement = (id, eleId) => {
    const { cardCell } = this.props

    if (id !== cardCell.uuid) return

    this.setState({elements: this.state.elements.filter(item => item.uuid !== eleId)})
  }

  cardAddElement = (id, element) => {
    const { cardCell } = this.props

    if (id !== cardCell.uuid) return
    if (window.GLOB.$lock) return

    window.GLOB.$lock = true
    setTimeout(() => {
      window.GLOB.$lock = false
    }, 200)

    const { elements } = this.state

    this.setState({elements: [...elements, element]})

    this.handleElement(element)
  }

  handleStyle = (element) => {
    let _style = element.style ? fromJS(element.style).toJS() : {}
    let options = ['font', 'border', 'padding', 'margin', 'background']

    if (element.eleType === 'button') {
      if (element.OpenType === 'form' && element.formType !== 'scan') {
        options = ['margin', 'float']
      } else {
        options.push('width', 'minHeight', 'float')
      }
      if (element.wrapStyle) {
        _style.float = element.wrapStyle.textAlign || 'left'
      }
    } else if (element.eleType === 'picture') {
      options = ['border', 'margin']
    } else if (element.eleType === 'text') {
      options[0] = 'font2'
    } else if (element.eleType === 'slider') {
      options = ['padding', 'margin']
    } else if (element.eleType === 'splitline') {
      options = ['padding', 'margin']
    }

    options.push('clear')

    this.setState({
      card: element
    })

    MKEmitter.emit('changeStyle', options, _style, this.getStyle)
  }

  getStyle = (style) => {
    const { card, elements } = this.state

    // if (card.eleType === 'button') {
    //   if ((style.paddingLeft || style.paddingRight) && !style.width) {
    //     style.width = 'auto'
    //   }
    // }

    let _card = this.resetCardStyle(card, style)

    let _elements = elements.map(cell => {
      if (cell.uuid === _card.uuid) return _card
      return cell
    })

    this.setState({
      elements: _elements
    }, () => {
      this.props.updateElement(_elements)
    })
  }

  resetCardStyle = (card, style) => {
    let _card = fromJS(card).toJS()

    if (['text', 'number', 'formula', 'currentDate', 'sequence', 'icon'].includes(_card.eleType)) {
      _card.style = style
      let line = _card.height || null

      if (['currentDate', 'sequence', 'icon'].includes(_card.eleType)) {
        line = 1
      }

      if (line) {
        let fontSize = 14
        let lineHeight = 1.5

        if (_card.style.fontSize) {
          fontSize = parseInt(_card.style.fontSize)
        }
        if (_card.style.lineHeight) {
          lineHeight = parseFloat(_card.style.lineHeight)
        }

        _card.innerHeight = fontSize * lineHeight * line
      } else {
        _card.innerHeight = 'auto'
      }
    } else if (_card.eleType === 'barcode') {
      _card.style = style

      let fontSize = 14

      if (_card.style.fontSize) {
        fontSize = parseInt(_card.style.fontSize)
      }

      _card.innerHeight = _card.barHeight + (_card.displayValue === 'true' ? fontSize + 2 : 0)
    } else if (_card.eleType === 'button') { // 拆分style
      _card.style = fromJS(style).toJS()

      if (style.float) {
        _card.wrapStyle = {textAlign: style.float}
        delete _card.style.float
      }
    } else {
      _card.style = style
    }

    return _card
  }

  /**
   * @description 按钮顺序调整
   */
  handleList = (list) => {
    this.setState({elements: list}, () => {
      this.props.updateElement(list)
    })
  }

  /**
   * @description 元素编辑，获取元素表单信息
   */
  handleElement = (card) => {
    const { cards, cardCell } = this.props

    if (card.eleType === 'button') {
      this.handleAction(card)
    } else {
      this.setState({
        visible: true,
        card: card,
        formlist: getCardCellForm(card, cards, cardCell)
      })
    }
  }

  /**
   * @description 按钮编辑，获取按钮表单信息
   */
  handleAction = (card) => {
    const { cards } = this.props

    let usefulFields = sessionStorage.getItem('permFuncField')
    if (usefulFields) {
      try {
        usefulFields = JSON.parse(usefulFields)
      } catch (e) {
        usefulFields = []
      }
    } else {
      usefulFields = []
    }

    let ableField = usefulFields.join(', ')
    let msg = `函数名称需以${ableField}等字符开始;`
    let functip = <div>
      <p style={{marginBottom: '5px'}}>{msg}</p>
    </div>

    let supId = ''
    if (cards.setting && cards.setting.supModule) {
      let pid = cards.setting.supModule[cards.setting.supModule.length - 1]
      if (pid && pid !== 'empty') {
        supId = pid
      } else {
        supId = ''
      }
    }

    let modules = MenuUtils.getSubModules(window.GLOB.customMenu.components, cards.uuid, supId)

    if (cards.subtype === 'basetable') {
      this.setState({
        actvisible: true,
        card: card,
        formlist: getBaseTableActionForm(card, functip, cards, usefulFields, modules)
      })
    } else {
      let anchors = MenuUtils.getAnchors(window.GLOB.customMenu.components, cards.uuid) || []
  
      this.setState({
        actvisible: true,
        card: card,
        formlist: getActionForm(card, functip, cards, usefulFields, modules, anchors)
      })
    }
  }

  /**
   * @description 取消保存，如果元素为新添元素，则从序列中删除
   */
  editModalCancel = () => {
    const { card, elements } = this.state
    let _elements = null

    if (card.focus) {
      _elements = elements.filter(item => item.uuid !== card.uuid)
    } else {
      _elements = elements
    }

    this.setState({
      card: null,
      elements: _elements,
      visible: false,
      actvisible: false
    }, () => {
      this.props.updateElement(_elements)
    })
  }

  /**
   * @description 元素修改后提交保存
   */
  handleSubmit = () => {
    const { elements } = this.state

    this.elementFormRef.handleConfirm().then(res => {
      let _elements = elements.map(cell => {
        if (cell.uuid === res.uuid) {
          res.style = cell.style || {}
          if (res.eleType === 'splitline' && (cell.eleType !== 'splitline' || cell.focus)) {
            res.style.paddingTop = '5px'
            res.style.paddingBottom = '5px'
          } else if (['text', 'number', 'formula', 'currentDate', 'sequence', 'icon'].includes(res.eleType)) {
            let line = res.height || null

            if (['currentDate', 'sequence', 'icon'].includes(res.eleType)) {
              line = 1
            }

            if (line) {
              let fontSize = 14
              let lineHeight = 1.5

              if (res.style && res.style.fontSize) {
                fontSize = parseInt(res.style.fontSize)
              }
              if (res.style && res.style.lineHeight) {
                lineHeight = parseFloat(res.style.lineHeight)
              }

              res.innerHeight = fontSize * lineHeight * line
            } else {
              res.innerHeight = 'auto'
            }

            if (res.eleType === 'text' && res.link && !res.style.color) {
              res.style.color = '#2440B3'
            }
          } else if (res.eleType === 'barcode') {
            let fontSize = 14
      
            if (res.style && res.style.fontSize) {
              fontSize = parseInt(res.style.fontSize)
            }
      
            res.innerHeight = res.barHeight + (res.displayValue === 'true' ? fontSize + 2 : 0)
          }
          
          return res
        }
        return cell
      })

      this.setState({
        elements: _elements,
        visible: false
      }, () => {
        this.props.updateElement(_elements)
      })
    })
  }

  /**
   * @description 元素修改后提交保存
   */
  handleActionSubmit = () => {
    const { elements } = this.state
    let color = { primary: '#1890ff', yellow: '#c49f47', orange: 'orange', danger: '#ff4d4f', green: '#26C281', dgreen: '#32c5d2', purple: '#8E44AD', cyan: '#13c2c2', gray: '#666666', default: 'rgba(0, 0, 0, 0.65)' }

    this.actionFormRef.handleConfirm().then(res => {
      let _elements = elements.map(cell => {
        if (cell.uuid === res.uuid) {
          res.eleType = cell.eleType || null
          res.style = cell.style || null
          res.modal = cell.modal || null
          res.wrapStyle = cell.wrapStyle || null
          // res = {...cell, ...res}

          // if (!res.control) {
          //   delete res.controlField
          //   delete res.controlVal
          // }
          
          // delete res.focus

          if (res.OpenType === 'form') {
            if (cell.OpenType !== 'form') {
              res.style = {}
            }
          } else if (res.class !== cell.class || res.show !== cell.show || !res.style) {
            let cl = res.class.replace('border-', '')
            let style = {}
            if (res.show === 'link' || res.show === 'icon') {
              style.color = color[cl]
              style.backgroundColor = 'transparent'
            } else {
              style.color = '#ffffff'
              style.backgroundColor = color[cl]
            }
            res.style = {...res.style, ...style}
          }

          return res
        }
        return cell
      })

      this.setState({
        elements: _elements,
        actvisible: false
      }, () => {
        this.props.updateElement(_elements)
      })
    })
  }

  /**
   * @description 按钮删除
   */
  deleteElement = (card) => {
    const { elements } = this.state
    let _this = this

    confirm({
      content: '确定删除元素吗？',
      onOk() {
        let _elements = elements.filter(item => item.uuid !== card.uuid)

        _this.setState({
          elements: _elements
        }, () => {
          _this.props.updateElement(_elements)
        })
      },
      onCancel() {}
    })
  }

  /**
   * @description 验证信息配置
   */
  profileAction = (element) => {
    this.setState({
      profVisible: true,
      card: element
    })
  }

  /**
   * @description 验证信息保存
   */
  verifySubmit = () => {
    const { elements, card } = this.state

    this.verifyRef.handleConfirm().then(res => {
      let _elements = elements.map(cell => {
        if (cell.uuid === card.uuid) {
          cell.verify = res
        }

        return cell
      })

      this.setState({
        elements: _elements,
        profVisible: false
      }, () => {
        this.props.updateElement(_elements)
      })
    })
  }

  handleSubConfig = (item) => {
    const { cards } = this.props
    const { appType } = this.state
    let btn = fromJS(item).toJS()

    if ((sessionStorage.getItem('style-control') && sessionStorage.getItem('style-control') === 'true')) return

    if (btn.OpenType === 'pop' || btn.execMode === 'pop') {
      if (!btn.modal) {
        btn.modal = {
          setting: { title: btn.label, width: appType === 'mob' ? 100 : 60, cols: '2', container: 'view', focus: '', finish: 'close', clickouter: 'unclose', display: 'modal' },
          tables: [],
          groups: [],
          fields: []
        }
      }

      MKEmitter.emit('changeModal', cards, btn)
    } else if (btn.OpenType === 'popview') {
      MKEmitter.emit('changePopview', cards, btn)
    } else if (btn.OpenType === 'innerpage' && btn.pageTemplate === 'linkpage') {
      MKEmitter.emit('changeEditMenu', {MenuID: btn.linkmenu})
    } else if (btn.OpenType === 'funcbutton' && (btn.funcType === 'copyurl' || btn.funcType === 'scan') && btn.linkmenu) {
      MKEmitter.emit('changeEditMenu', {MenuID: btn.linkmenu})
    } else {
      this.handleElement(item)
    }
  }

  handleSave = (componentId, btnId, modal) => {
    const { cards } = this.props
    const { elements } = this.state

    if (cards.uuid !== componentId) return
    
    let _index = elements.findIndex(cell => cell.uuid === btnId)

    if (_index === -1) return

    let _elements = elements.map(cell => {
      if (cell.uuid === btnId) {
        cell.modal = modal
      }

      return cell
    })

    this.setState({
      elements: _elements
    }, () => {
      this.props.updateElement(_elements)
    })
  }

  updateMarks = (card) => {
    const { elements } = this.state

    let _elements = elements.map(cell => {
      if (cell.uuid === card.uuid) return card
      return cell
    })

    this.setState({
      elements: _elements
    }, () => {
      this.props.updateElement(_elements)
    })
  }

  dropButton = (id) => {
    const { cards, cardCell } = this.props

    if (!cards.action) return
    if (cardCell.type === 'custom') return

    let index = cards.action.findIndex(item => item.uuid === id)

    if (index === -1) return

    let btn = cards.action[index]
    btn.eleType = 'button'

    if (!btn.width) {
      btn.width = 12
    }
    if (btn.color) {
      btn.style = btn.style || {}
      btn.style.color = '#ffffff'
      if (btn.color === 'primary') {
        btn.style.backgroundColor = '#1677ff'
      } else if (btn.color === 'danger') {
        btn.style.backgroundColor = '#ff3141'
      } else if (btn.color === 'warning') {
        btn.style.backgroundColor = '#ff8f1f'
      } else if (btn.color === 'success') {
        btn.style.backgroundColor = '#00b578'
      } else if (btn.color === 'light') {
        btn.style.backgroundColor = '#cccccc'
      }
    }

    let _elements = [...this.state.elements, btn]

    this.setState({
      elements: _elements
    }, () => {
      this.props.updateElement(_elements, btn)
    })
  }

  getVerify = (card) => {
    const { cards } = this.props

    if (!card) return null

    if (['pop', 'prompt', 'exec', 'form'].includes(card.OpenType)) {
      return <VerifyCard
        card={card}
        config={cards}
        columns={cards.columns}
        wrappedComponentRef={(inst) => this.verifyRef = inst}
      />
    } else if (card.OpenType === 'excelIn') {
      return <VerifyExcelIn
        card={card}
        columns={cards.columns}
        wrappedComponentRef={(inst) => this.verifyRef = inst}
      />
    } else if (card.OpenType === 'excelOut') {
      return <VerifyExcelOut
        card={card}
        config={cards}
        wrappedComponentRef={(inst) => this.verifyRef = inst}
      />
    } else if (card.OpenType === 'funcbutton' && card.funcType === 'print') {
      return <VerifyPrint
        card={card}
        columns={cards.columns}
        wrappedComponentRef={(inst) => this.verifyRef = inst}
      />
    }
  }

  render() {
    const { cards, cardCell } = this.props
    const { elements, visible, actvisible, profVisible, card, record } = this.state

    return (
      <div className="model-menu-card-cell-list">
        <DragElement
          list={elements}
          parent={cardCell}
          fields={cards.columns}
          updateMarks={this.updateMarks}
          handleList={this.handleList}
          handleMenu={this.handleElement}
          handleStyle={this.handleStyle}
          dropButton={this.dropButton}
          profileAction={this.profileAction}
          handleSubConfig={this.handleSubConfig}
          deleteMenu={this.deleteElement}
        />
        <div onDoubleClick={(e) => e.stopPropagation()}>
          {/* 编辑按钮：复制、编辑 */}
          <Modal
            title="编辑元素"
            visible={visible}
            width={850}
            maskClosable={false}
            onCancel={this.editModalCancel}
            onOk={this.handleSubmit}
            destroyOnClose
          >
            <ElementForm
              card={card}
              formlist={this.state.formlist}
              inputSubmit={this.handleSubmit}
              config={cards}
              wrappedComponentRef={(inst) => this.elementFormRef = inst}
            />
          </Modal>
          {/* 编辑按钮：复制、编辑 */}
          <Modal
            title="按钮·编辑"
            visible={actvisible}
            width={920}
            maskClosable={false}
            onCancel={this.editModalCancel}
            footer={[
              record && record.intertype === 'inner' ? <CreateFunc key="create" ref="btnCreatFunc" trigger={this.creatFunc}/> : null,
              <Button key="cancel" onClick={this.editModalCancel}>取消</Button>,
              <Button key="confirm" type="primary" onClick={this.handleActionSubmit}>确定</Button>
            ]}
            destroyOnClose
          >
            <ActionForm
              type={cards.type === 'balcony' ? '' : 'card'}
              card={card}
              formlist={this.state.formlist}
              inputSubmit={this.handleActionSubmit}
              setting={cards.setting}
              updRecord={(record) => this.setState({record: fromJS(record).toJS()})}
              wrappedComponentRef={(inst) => this.actionFormRef = inst}
            />
          </Modal>
          {/* 按钮使用系统存储过程时，验证信息模态框 */}
          <Modal
            wrapClassName="mk-pop-modal"
            visible={profVisible}
            width={'90vw'}
            maskClosable={false}
            okText="提交"
            onOk={this.verifySubmit}
            onCancel={() => {
              if (this.verifyRef.handleCancel) {
                this.verifyRef.handleCancel().then(() => {
                  this.setState({ profVisible: false })
                })
              } else {
                this.setState({ profVisible: false })
              }
            }}
            destroyOnClose
          >
            {this.getVerify(card)}
          </Modal>
        </div>
      </div>
    )
  }
}

export default CardCellComponent